require("lua/utils/string_utils.lua")

local base_drone = require("lua/units/air/base_drone.lua")
class 'control_drone' ( base_drone )

function control_drone:__init()
	base_drone.__init(self,self)
end

function control_drone:FillInitialParams()
    self.team = EntityService:GetTeam( self.entity );
    self.action_tag_name = "action##" .. tostring( self.entity );
    self.search_radius = self.data:GetFloat("search_radius")

    self.follow_distance = 10.0;
end

function control_drone:OnDroneLifting()
	QueueEvent( "FadeEntityInRequest", self.entity, 0.5 )
end

function control_drone:OnDroneLanding()
	QueueEvent( "FadeEntityOutRequest", self.entity, 0.5 )
end

function control_drone:OnInit()
    self:FillInitialParams();

    self.request_created = false;
    self.found_target = INVALID_ID;

    self.fsm = self:CreateStateMachine();
    self.fsm:AddState("control", { execute="OnControlExecute", exit="OnControlExit"});

	WeaponService:UpdateWeaponStatComponent( self.entity, self.entity )

	QueueEvent( "FadeEntityOutRequest", self.entity, 0.5 )
end

function control_drone:OnLoad()
    self:FillInitialParams();
end

function control_drone:FindActionTarget()
    if not self.request_created then
        self.request_created = true

        local owner = self:GetDroneOwnerTarget()

        self:RegisterHandler(owner, "TargetHasChangedEvent", "OnTargetHasChangedEvent")
        UnitService:CreateFindTargetRequest( owner, self.action_tag_name, self.search_radius, "hostility", "ground_unit" );
    end

    if self.found_target ~= INVALID_ID then
        if EntityService:IsAlive(self.found_target) then
            self.fsm:ChangeState("control")
        else
            self.found_target = INVALID_ID
        end
    end

    return self.found_target;
end

function control_drone:OnTargetHasChangedEvent( evt )
    if evt:GetTag() ~= self.action_tag_name then
        return
    end

    self.found_target = evt:GetTarget();
end

function control_drone:OnControlExecute(state)
    local target = self:GetDroneActionTarget();
    if not HealthService:IsAlive( target ) then
        self:SetTargetActionFinished();
        state:Exit();
        return
    end
    local distance = EntityService:GetDistance2DBetween( self.entity, target );
	
    local is_shooting =  WeaponService:IsShooting( self.entity );
    if is_shooting and distance > self.follow_distance then
        WeaponService:StopShoot(self.entity);
    end

    if not is_shooting and distance <= self.follow_distance then
        WeaponService:StartShoot(self.entity);
        EntityService:SetTeam( target, "player" );
        self:SetTargetActionFinished();
        state:Exit();
        return
    end
end

function control_drone:OnControlExit()
    WeaponService:StopShoot(self.entity);
end

function control_drone:OnDroneTargetAction( target )
    return false;
end

return control_drone;